/// <reference types="node" />
import { Buffer } from 'buffer';
import { FieldDescription, Uint, Uint64, NitroSigner } from '@cerc-io/nitro-util';
import { Signature } from '../crypto/signatures';
import { Destination } from '../types/destination';
import { Address } from '../types/types';
import { Funds } from '../types/funds';
import { Allocation } from './state/outcome/allocation';
import { SignedState } from './state/signedstate';
import { FixedPart, State, ConstructorOptions as FixedPartConstructorOptions } from './state/state';
interface ConstructorOptions extends FixedPartConstructorOptions {
    id?: Destination;
    myIndex?: Uint;
    onChainFunding?: Funds;
    fixedPart?: FixedPart;
    signedStateForTurnNum?: Map<Uint64, SignedState>;
    latestSupportedStateTurnNum?: Uint64;
}
export declare class Channel extends FixedPart {
    id: Destination;
    myIndex: Uint;
    onChainFunding: Funds;
    signedStateForTurnNum: Map<Uint64, SignedState>;
    latestSupportedStateTurnNum: Uint64;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): Channel;
    toJSON(): any;
    constructor(params: ConstructorOptions);
    static new(s: State, myIndex: Uint): Channel;
    marshalJSON(): Buffer;
    unmarshalJSON(data: Buffer): void;
    myDestination(): Destination;
    clone(): Channel;
    preFundState(): State;
    signedPreFundState(): SignedState;
    postFundState(): State;
    signedPostFundState(): SignedState;
    preFundSignedByMe(): boolean;
    postFundSignedByMe(): boolean;
    preFundComplete(): boolean;
    postFundComplete(): boolean;
    finalSignedByMe(): boolean;
    finalCompleted(): boolean;
    hasSupportedState(): boolean;
    latestSupportedState(): State;
    latestSignedState(): SignedState;
    total(): Funds;
    affords(allocationMap: Map<Address, Allocation>, fundingMap: Funds): boolean;
    addStateWithSignature(s: State, sig: Signature): boolean;
    addSignedState(ss: SignedState): boolean;
    signAndAddPrefund(signer: NitroSigner): Promise<SignedState>;
    signAndAddPostfund(signer: NitroSigner): Promise<SignedState>;
    signAndAddState(s: State, signer: NitroSigner): Promise<SignedState>;
}
export {};
