/// <reference types="node" />
import { Buffer } from 'buffer';
import { State as NitroState } from '@statechannels/nitro-protocol';
import { FieldDescription, NitroSigner, Uint64 } from '@cerc-io/nitro-util';
import * as nc from '../../crypto/signatures';
import { Address } from '../../types/types';
import { Destination } from '../../types/destination';
import { Exit } from './outcome/exit';
export type Signature = nc.Signature;
export interface ConstructorOptions {
    participants?: Address[] | null;
    channelNonce?: Uint64;
    appDefinition?: Address;
    challengeDuration?: number;
}
export declare class FixedPart {
    participants: Address[] | null;
    channelNonce: Uint64;
    appDefinition: Address;
    challengeDuration: number;
    constructor(params: ConstructorOptions);
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): FixedPart;
    toJSON(): any;
    channelId(): Destination;
    clone(): FixedPart;
    validate(): void;
}
export declare class VariablePart {
    appData: Buffer | null;
    outcome: Exit;
    turnNum: Uint64;
    isFinal: boolean;
    constructor(params: {
        appData?: Buffer | null;
        outcome?: Exit;
        turnNum?: Uint64;
        isFinal?: boolean;
    });
}
export declare class State {
    participants: Address[] | null;
    channelNonce: Uint64;
    appDefinition: Address;
    challengeDuration: number;
    appData: Buffer | null;
    outcome: Exit;
    turnNum: Uint64;
    isFinal: boolean;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): State;
    toJSON(): any;
    constructor(params: {
        participants?: Address[] | null;
        channelNonce?: Uint64;
        appDefinition?: Address;
        challengeDuration?: number;
        appData?: Buffer | null;
        outcome?: Exit;
        turnNum?: Uint64;
        isFinal?: boolean;
    });
    fixedPart(): FixedPart;
    variablePart(): VariablePart;
    channelId(): Destination;
    encode(): Buffer;
    hash(): string;
    sign(signer: NitroSigner): Promise<Signature>;
    recoverSigner(sig: Signature): Address;
    equal(r: State): boolean;
    validate(): void;
    clone(): State;
    _getNitroState(): NitroState;
}
export declare function stateFromFixedAndVariablePart(f: FixedPart, v: VariablePart): State;
