import type { ReadChannel } from '@cerc-io/ts-channel';
import { ChainTransaction, Objective } from '../../../protocols/interfaces';
import { Address } from '../../../types/types';
import { Destination } from '../../../types/destination';
export interface ChainEvent {
    channelID(): Destination;
}
interface CommonEventConstructorOptions {
    _channelID?: Destination;
    blockNum?: string;
}
declare class CommonEvent implements ChainEvent {
    private _channelID;
    blockNum: string;
    constructor(params: CommonEventConstructorOptions);
    channelID(): Destination;
}
interface AssetAndAmountConstructorOptions {
    assetAddress?: Address;
    assetAmount?: bigint;
}
declare class AssetAndAmount {
    assetAddress: Address;
    assetAmount?: bigint;
    constructor(params: AssetAndAmountConstructorOptions);
    string(): string;
}
export declare class DepositedEvent extends CommonEvent {
    nowHeld?: bigint;
    assetAndAmount: AssetAndAmount;
    constructor(params: {
        nowHeld?: bigint;
    } & CommonEventConstructorOptions, assetAndAmountParams: AssetAndAmountConstructorOptions);
    static newDepositedEvent(channelId: Destination, blockNum: string, assetAddress: Address, assetAmount?: bigint, nowHeld?: bigint): DepositedEvent;
    string(): string;
}
export interface ChainEventHandler {
    updateWithChainEvent(event: ChainEvent): Objective;
}
export interface ChainService {
    eventFeed(): ReadChannel<ChainEvent>;
    sendTransaction(tx: ChainTransaction): Promise<void>;
    getConsensusAppAddress(): Address;
    getVirtualPaymentAppAddress(): Address;
    getChainId(): Promise<bigint>;
    close(): Promise<void>;
}
export declare class ConcludedEvent extends CommonEvent {
    string(): string;
}
export declare class AllocationUpdatedEvent extends CommonEvent {
    assetAndAmount: AssetAndAmount;
    string(): string;
    static newAllocationUpdatedEvent(channelId: Destination, blockNum: string, assetAddress: Address, assetAmount: bigint | undefined): AllocationUpdatedEvent;
    constructor(params: CommonEventConstructorOptions, assetAndAmountParams: AssetAndAmountConstructorOptions);
}
export {};
