import { ethers, providers } from 'ethers';
import debug from 'debug';
import { WaitGroup } from '@jpwilliams/waitgroup';
import type { ReadChannel, ReadWriteChannel } from '@cerc-io/ts-channel';
import { Context } from '@cerc-io/nitro-util';
import { ChainService, ChainEvent } from './chainservice';
import { ChainTransaction } from '../../../protocols/interfaces';
import { Address } from '../../../types/types';
import { NitroAdjudicator } from './adjudicator/nitro-adjudicator';
interface EthChain {
    provider: ethers.providers.BaseProvider;
    chainID(): Promise<bigint>;
}
export declare class EthChainService implements ChainService {
    private chain;
    private na;
    private naAddress;
    private consensusAppAddress;
    private virtualPaymentAppAddress;
    private txSigner;
    private out;
    private logger;
    private ctx;
    private cancel;
    private wg?;
    constructor(chain: EthChain, na: NitroAdjudicator, naAddress: string, consensusAppAddress: string, virtualPaymentAppAddress: string, txSigner: ethers.Signer, out: ReadWriteChannel<ChainEvent>, logger: debug.Debugger, ctx: Context, cancel: () => void, wg: WaitGroup);
    static newEthChainService(chainUrl: string, chainPk: string, naAddress: Address, caAddress: Address, vpaAddress: Address, logDestination?: WritableStream): Promise<EthChainService>;
    static newEthChainServiceWithProvider(provider: providers.JsonRpcProvider, naAddress: Address, caAddress: Address, vpaAddress: Address, logDestination?: WritableStream): Promise<EthChainService>;
    private static _newEthChainService;
    private listenForErrors;
    private defaultTxOpts;
    sendTransaction(tx: ChainTransaction): Promise<void>;
    private dispatchChainEvents;
    private listenForLogs;
    private subscribeForLogs;
    eventFeed(): ReadChannel<ChainEvent>;
    getConsensusAppAddress(): Address;
    getVirtualPaymentAppAddress(): Address;
    getChainId(): Promise<bigint>;
    close(): Promise<void>;
}
export {};
