import debug from 'debug';
import type { ReadChannel, ReadWriteChannel } from '@cerc-io/ts-channel';
import type { Libp2p } from '@cerc-io/libp2p';
import type { PrivateKey } from '@libp2p/interface-keys';
import type { PeerId } from '@libp2p/interface-peer-id';
import type { Peer } from '@cerc-io/peer';
import { SafeSyncMap } from '../../../../internal/safesync/safesync';
import { Message } from '../../../../protocols/messages';
import { Address } from '../../../../types/types';
import { MessageService } from '../messageservice';
export interface BasicPeerInfo {
    id: PeerId;
    address: Address;
}
export interface PeerInfo {
    port: number;
    id: PeerId;
    address: Address;
    ipAddress: string;
}
interface ConstructorOptions {
    toEngine: ReadWriteChannel<Message>;
    peers: SafeSyncMap<BasicPeerInfo>;
    me: Address;
    newPeerInfo: ReadWriteChannel<BasicPeerInfo>;
    logger: debug.Debugger;
    key?: PrivateKey;
    p2pHost?: Libp2p;
}
export declare class P2PMessageService implements MessageService {
    private toEngine?;
    private peers?;
    private me;
    private key?;
    private p2pHost?;
    private newPeerInfo?;
    private logger;
    private sentInfoToPeer;
    private peer?;
    constructor(params: ConstructorOptions);
    static newMessageService(me: Address, peer: Peer, logWriter?: WritableStream): Promise<P2PMessageService>;
    id(): Promise<PeerId>;
    private exchangeInfoWithConnectedPeers;
    private handleChangeProtocols;
    private handlePeerConnect;
    private exchangePeerInfo;
    private msgStreamHandler;
    private sendPeerInfo;
    private receivePeerInfo;
    send(msg: Message): Promise<void>;
    private checkError;
    out(): ReadChannel<Message>;
    close(): Promise<void>;
    peerInfoReceived(): ReadChannel<BasicPeerInfo>;
    addPeers(peers: PeerInfo[]): Promise<void>;
    addPeerByMultiaddr(clientAddress: Address, multiaddrString: string): Promise<void>;
    private connectAndSendPeerInfos;
    isPeerDialable(peerAddress: string): Promise<[boolean, string]>;
}
export {};
