/// <reference types="node" />
import { Buffer } from 'buffer';
import type { NitroSigner } from '@cerc-io/nitro-util';
import { Store } from './store';
import { Objective } from '../../../protocols/interfaces';
import { Channel } from '../../../channel/channel';
import { ConsensusChannel } from '../../../channel/consensus-channel/consensus-channel';
import { VoucherInfo } from '../../../payments/vouchers';
import { SafeSyncMap } from '../../../internal/safesync/safesync';
import { ObjectiveId } from '../../../protocols/messages';
import { Address } from '../../../types/types';
import { Destination } from '../../../types/destination';
export declare class MemStore implements Store {
    objectives?: SafeSyncMap<Buffer>;
    channels?: SafeSyncMap<Buffer>;
    consensusChannels?: SafeSyncMap<Buffer>;
    channelToObjective?: SafeSyncMap<ObjectiveId>;
    vouchers?: SafeSyncMap<Buffer>;
    signer?: NitroSigner;
    address: string;
    static newMemStore(signer: NitroSigner): Promise<MemStore>;
    close(): void;
    getAddress(): Address;
    getChannelSigner(): NitroSigner;
    getObjectiveById(id: ObjectiveId): Objective;
    setObjective(obj: Objective): void;
    setChannel(ch: Channel): void;
    destroyChannel(id: Destination): void;
    setConsensusChannel(ch: ConsensusChannel): void;
    destroyConsensusChannel(id: Destination): void;
    getChannelById(id: Destination): [Channel, boolean];
    private _getChannelById;
    getChannelsByIds(ids: Destination[]): Channel[];
    getChannelsByAppDefinition(appDef: Address): Channel[];
    getChannelsByParticipant(participant: Address): Channel[];
    getConsensusChannelById(id: Destination): ConsensusChannel;
    getConsensusChannel(counterparty: Address): [ConsensusChannel | undefined, boolean];
    getAllConsensusChannels(): ConsensusChannel[];
    getObjectiveByChannelId(channelId: Destination): [Objective | undefined, boolean];
    populateChannelData(obj: Objective): void;
    releaseChannelFromOwnership(channelId: Destination): void;
    setVoucherInfo(channelId: Destination, v: VoucherInfo): void;
    getVoucherInfo(channelId: Destination): [VoucherInfo | undefined, boolean];
    removeVoucherInfo(channelId: Destination): void;
}
export declare function decodeObjective(id: ObjectiveId, data: Buffer): Objective;
export declare function contains<T extends Destination | ObjectiveId>(col: T[], item: T): boolean;
