import type { ReadChannel, ReadWriteChannel } from '@cerc-io/ts-channel';
import { MessageService } from './engine/messageservice/messageservice';
import { ChainService } from './engine/chainservice/chainservice';
import { Store } from './engine/store/store';
import { PolicyMaker } from './engine/policy-maker';
import { Address } from '../types/types';
import { ObjectiveId } from '../protocols/messages';
import { Voucher } from '../payments/vouchers';
import { MetricsApi } from './engine/metrics';
import { Exit } from '../channel/state/outcome/exit';
import { ObjectiveResponse as DirectFundObjectiveResponse } from '../protocols/directfund/directfund';
import { ObjectiveResponse as VirtualFundObjectiveResponse } from '../protocols/virtualfund/virtualfund';
import { Destination } from '../types/destination';
import { PaymentChannelInfo, LedgerChannelInfo } from './query/types';
export declare class Node {
    private engine;
    address: Address;
    private channelNotifier?;
    private completedObjectivesForRPC?;
    private completedObjectives?;
    private failedObjectives?;
    private _receivedVouchers?;
    private chainId?;
    private store?;
    private vm?;
    private logger;
    private cancelEventHandler?;
    private wg?;
    static new(messageService: MessageService, chainservice: ChainService, store: Store, logDestination: WritableStream | undefined, policymaker: PolicyMaker, metricsApi?: MetricsApi): Promise<Node>;
    createLedgerChannel(counterparty: Address, challengeDuration: number, outcome: Exit): Promise<DirectFundObjectiveResponse>;
    closeLedgerChannel(channelId: Destination): Promise<ObjectiveId>;
    createPaymentChannel(intermediaries: Address[], counterParty: Address, challengeDuration: number, outcome: Exit): Promise<VirtualFundObjectiveResponse>;
    closePaymentChannel(channelId: Destination): Promise<ObjectiveId>;
    pay(channelId: Destination, amount?: bigint): Promise<void>;
    getPaymentChannel(id: Destination): Promise<PaymentChannelInfo>;
    private handleEngineEvents;
    objectiveCompleteChan(id: ObjectiveId): ReadWriteChannel<null>;
    stopEventHandler(): Promise<void>;
    close(): Promise<void>;
    receivedVouchers(): ReadChannel<Voucher>;
    getPaymentChannelsByLedger(ledgerId: Destination): Promise<PaymentChannelInfo[]>;
    getAllLedgerChannels(): Promise<LedgerChannelInfo[]>;
    getLedgerChannel(id: Destination): Promise<LedgerChannelInfo>;
    handleError(err: Error): Promise<void>;
    sentVouchers(): ReadChannel<Voucher>;
}
