import { ReadWriteChannel } from '@cerc-io/ts-channel';
import { FieldDescription, NitroSigner } from '@cerc-io/nitro-util';
import { Destination } from '../../types/destination';
import { ConsensusChannel } from '../../channel/consensus-channel/consensus-channel';
import * as channel from '../../channel/channel';
import { ObjectiveRequest as ObjectiveRequestInterface, Objective as ObjectiveInterface, SideEffects, WaitingFor, Storable, ObjectiveStatus } from '../interfaces';
import { ObjectiveId, ObjectivePayload } from '../messages';
import { Address } from '../../types/types';
import { ChainEvent } from '../../node/engine/chainservice/chainservice';
type GetConsensusChannel = (channelId: Destination) => (ConsensusChannel | undefined) | Promise<ConsensusChannel | undefined>;
export declare class Objective implements ObjectiveInterface {
    status: ObjectiveStatus;
    c?: channel.Channel;
    private finalTurnNum;
    private transactionSubmitted;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): Objective;
    toJSON(): any;
    constructor(params: {
        status?: ObjectiveStatus;
        c?: channel.Channel;
        finalTurnNum?: number;
        transactionSubmitted?: boolean;
    });
    static newObjective(request: ObjectiveRequest, preApprove: boolean, getConsensusChannel: GetConsensusChannel): Promise<Objective>;
    static constructObjectiveFromPayload(p: ObjectivePayload, preapprove: boolean, getConsensusChannel: GetConsensusChannel): Promise<Objective>;
    id(): ObjectiveId;
    approve(): ObjectiveInterface;
    reject(): [ObjectiveInterface, SideEffects];
    ownsChannel(): Destination;
    getStatus(): ObjectiveStatus;
    related(): Storable[];
    update(p: ObjectivePayload): ObjectiveInterface;
    updateWithChainEvent(event: ChainEvent): ObjectiveInterface;
    crank(signer: NitroSigner): Promise<[Objective, SideEffects, WaitingFor]>;
    clone(): Objective;
    private fullyWithdrawn;
    private otherParticipants;
}
export declare function isDirectDefundObjective(id: ObjectiveId): boolean;
export declare class ObjectiveRequest implements ObjectiveRequestInterface {
    channelId: Destination;
    private objectiveStarted?;
    constructor(params: {
        channelId?: Destination;
        objectiveStarted: ReadWriteChannel<void>;
    });
    static newObjectiveRequest(channelId: Destination): ObjectiveRequest;
    id(address: Address, chainId?: bigint): ObjectiveId;
    waitForObjectiveToStart(): Promise<void>;
    signalObjectiveStarted(): void;
}
export {};
