/// <reference types="node" />
import { Buffer } from 'buffer';
import type { ReadWriteChannel } from '@cerc-io/ts-channel';
import { FieldDescription, NitroSigner, Uint64 } from '@cerc-io/nitro-util';
import { Exit } from '../../channel/state/outcome/exit';
import { Address } from '../../types/types';
import { Funds } from '../../types/funds';
import { ObjectiveId, ObjectivePayload } from '../messages';
import { ObjectiveStatus, ObjectiveRequest as ObjectiveRequestInterface, Objective as ObjectiveInterface, SideEffects, WaitingFor, Storable } from '../interfaces';
import * as channel from '../../channel/channel';
import { ConsensusChannel } from '../../channel/consensus-channel/consensus-channel';
import { Destination } from '../../types/destination';
import { ChainEvent } from '../../node/engine/chainservice/chainservice';
export declare function fundOnChainEffect(cId: Destination, asset: string, amount: Funds): string;
interface GetChannelsByParticipantFunction {
    (participant: Address): channel.Channel[] | Promise<channel.Channel[]>;
}
interface GetTwoPartyConsensusLedgerFunction {
    (counterparty: Address): [ConsensusChannel | undefined, boolean] | Promise<[ConsensusChannel | undefined, boolean]>;
}
export declare class Objective implements ObjectiveInterface {
    status: ObjectiveStatus;
    c?: channel.Channel;
    private myDepositSafetyThreshold;
    private myDepositTarget;
    private fullyFundedThreshold;
    private latestBlockNumber;
    private transactionSubmitted;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): Objective;
    toJSON(): any;
    constructor(params: {
        status?: ObjectiveStatus;
        c?: channel.Channel;
        myDepositSafetyThreshold?: Funds;
        myDepositTarget?: Funds;
        fullyFundedThreshold?: Funds;
        latestBlockNumber?: number;
        transactionSubmitted?: boolean;
    });
    static newObjective(request: ObjectiveRequest, preApprove: boolean, myAddress: Address, chainId: bigint | undefined, getChannels: GetChannelsByParticipantFunction, getTwoPartyConsensusLedger: GetTwoPartyConsensusLedgerFunction): Promise<Objective>;
    static constructFromPayload(preApprove: boolean, op: ObjectivePayload, myAddress: Address): Objective;
    ownsChannel(): Destination;
    getStatus(): ObjectiveStatus;
    createConsensusChannel(): ConsensusChannel;
    id(): ObjectiveId;
    approve(): ObjectiveInterface;
    reject(): [ObjectiveInterface, SideEffects];
    update(p: ObjectivePayload): ObjectiveInterface;
    updateWithChainEvent(event: ChainEvent): ObjectiveInterface;
    private otherParticipants;
    /**
     * Crank inspects the extended state and declares a list of Effects to be executed.
     * It's like a state machine transition function where the finite/enumerable state is returned
     * (computed from the extended state) rather than being independent of the extended state;
     * and where there is only one type of event ("the crank") with no data on it at all.
     */
    crank(signer: NitroSigner): Promise<[Objective, SideEffects, WaitingFor]>;
    related(): Storable[];
    private fundingComplete;
    private safeToDeposit;
    private amountToDeposit;
    /**
    * clone returns a deep copy of the receiver.
    */
    private clone;
    marshalJSON(): Buffer;
    unmarshalJSON(b: Buffer): void;
}
export type ObjectiveResponse = {
    id: ObjectiveId;
    channelId: Destination;
};
export declare function isDirectFundObjective(id: ObjectiveId): boolean;
export declare class ObjectiveRequest implements ObjectiveRequestInterface {
    counterParty: Address;
    challengeDuration: number;
    outcome: Exit;
    appDefinition: Address;
    appData: Buffer | null;
    nonce: Uint64;
    private objectiveStarted?;
    constructor(params: {
        counterParty: Address;
        challengeDuration: number;
        outcome: Exit;
        appDefinition: Address;
        appData?: Buffer | null;
        nonce: Uint64;
        objectiveStarted?: ReadWriteChannel<void>;
    });
    static newObjectiveRequest(counterparty: Address, challengeDuration: number, outcome: Exit, nonce: Uint64, appDefinition: Address): ObjectiveRequest;
    signalObjectiveStarted(): void;
    waitForObjectiveToStart(): Promise<void>;
    id(myAddress: Address, chainId?: bigint): ObjectiveId;
    response(myAddress: Address, chainId?: bigint): ObjectiveResponse;
}
export {};
