import { NitroSigner } from '@cerc-io/nitro-util';
import { Message, ObjectiveId, ObjectivePayload } from './messages';
import { Proposal, SignedProposal } from '../channel/consensus-channel/consensus-channel';
import { Address } from '../types/types';
import { Destination } from '../types/destination';
import { Funds } from '../types/funds';
import { SignedState } from '../channel/state/signedstate';
export declare const errNotApproved: Error;
export interface ChainTransaction {
    channelId: () => Destination;
}
declare class ChainTransactionBase implements ChainTransaction {
    private _channelId;
    constructor(channelId: Destination);
    channelId(): Destination;
}
export declare class DepositTransaction extends ChainTransactionBase implements ChainTransaction {
    deposit: Funds;
    constructor(params: {
        channelId: Destination;
        deposit?: Funds;
    });
    static newDepositTransaction(channelId: Destination, deposit: Funds): DepositTransaction;
}
export declare class WithdrawAllTransaction extends ChainTransactionBase implements ChainTransaction {
    signedState: SignedState;
    constructor(params: {
        channelId: Destination;
        signedState?: SignedState;
    });
    static newWithdrawAllTransaction(channelId: Destination, signedState: SignedState): WithdrawAllTransaction;
}
export declare class SideEffects {
    messagesToSend: Message[];
    transactionsToSubmit: ChainTransaction[];
    proposalsToProcess: Proposal[];
    constructor(params: {
        messagesToSend?: Message[];
        transactionsToSubmit?: ChainTransaction[];
        proposalsToProcess?: Proposal[];
    });
    merge(other: SideEffects): void;
}
export type WaitingFor = string;
export interface Storable {
    toJSON(): any;
}
export interface Objective extends Storable {
    id(): ObjectiveId;
    approve(): Objective;
    reject(): [Objective, SideEffects];
    update(payload: ObjectivePayload): Objective;
    crank(signer: NitroSigner): Promise<[Objective, SideEffects, WaitingFor]>;
    related(): Storable[];
    ownsChannel(): Destination;
    getStatus(): ObjectiveStatus;
}
export interface ProposalReceiver extends Objective {
    receiveProposal(signedProposal: SignedProposal): ProposalReceiver;
}
export declare enum ObjectiveStatus {
    Unapproved = 0,
    Approved = 1,
    Rejected = 2,
    Completed = 3
}
export interface ObjectiveRequest {
    id(address: Address, chainId?: bigint): ObjectiveId;
    waitForObjectiveToStart(): void;
    signalObjectiveStarted(): void;
}
export {};
