/// <reference types="node" />
import { Buffer } from 'buffer';
import { FieldDescription, Uint64 } from '@cerc-io/nitro-util';
import { SignedProposal, Proposal } from '../channel/consensus-channel/consensus-channel';
import { Voucher } from '../payments/vouchers';
import { Address } from '../types/types';
export type ObjectiveId = string;
export type ObjectivePayload = {
    payloadData: Buffer;
    objectiveId: ObjectiveId;
    type: PayloadType;
};
export type PayloadType = string;
interface MessageSummary {
    to: string;
    from: string;
    payloadSummaries: ObjectivePayloadSummary[];
    proposalSummaries: ProposalSummary[];
    payments: PaymentSummary[];
    rejectedObjectives: string[];
}
interface ObjectivePayloadSummary {
    objectiveId: string;
    type: string;
    payloadDataSize: number;
}
interface ProposalSummary {
    objectiveId: string;
    ledgerId: string;
    proposalType: string;
    turnNum: Uint64;
}
interface PaymentSummary {
    amount: Uint64;
    channelId: string;
}
export declare function getProposalObjectiveId(p: Proposal): ObjectiveId;
export declare class Message {
    to: Address;
    from: Address;
    objectivePayloads: ObjectivePayload[] | null;
    ledgerProposals: SignedProposal[] | null;
    payments: Voucher[] | null;
    rejectedObjectives: ObjectiveId[] | null;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): Message;
    toJSON(): any;
    constructor(params: {
        to?: Address;
        from?: Address;
        objectivePayloads?: ObjectivePayload[];
        ledgerProposals?: SignedProposal[];
        payments?: Voucher[];
        rejectedObjectives?: ObjectiveId[];
    });
    static createObjectivePayloadMessage(id: ObjectiveId, p: any, payloadType: PayloadType, ...recipients: Address[]): Message[];
    static createRejectionNoticeMessage(oId: ObjectiveId, ...recipients: Address[]): Message[];
    static createSignedProposalMessage(recipient: Address, ...proposals: SignedProposal[]): Message;
    static createVoucherMessage(voucher: Voucher, ...recipients: Address[]): Message[];
    serialize(): string;
    summarize(): MessageSummary;
}
export declare function deserializeMessage(s: string): Message;
export {};
