/// <reference types="node" />
import { Buffer } from 'buffer';
import { ReadWriteChannel } from '@cerc-io/ts-channel';
import { FieldDescription, NitroSigner, Uint } from '@cerc-io/nitro-util';
import { Destination } from '../../types/destination';
import { Address } from '../../types/types';
import * as channel from '../../channel/channel';
import { VirtualChannel } from '../../channel/virtual';
import { ConsensusChannel, SignedProposal } from '../../channel/consensus-channel/consensus-channel';
import { ObjectiveRequest as ObjectiveRequestInterface, Objective as ObjectiveInterface, SideEffects, WaitingFor, Storable, ObjectiveStatus, ProposalReceiver } from '../interfaces';
import { ObjectiveId, ObjectivePayload } from '../messages';
import { SignedState } from '../../channel/state/signedstate';
import { SingleAssetExit } from '../../channel/state/outcome/exit';
import { FixedPart } from '../../channel/state/state';
export declare const ObjectivePrefix = "VirtualDefund-";
type GetChannelByIdFunction = (id: Destination) => [channel.Channel | undefined, boolean] | Promise<[channel.Channel | undefined, boolean]>;
type GetTwoPartyConsensusLedgerFunction = (counterparty: Address) => [ConsensusChannel | undefined, boolean] | Promise<[ConsensusChannel | undefined, boolean]>;
export declare function getSignedStatePayload(b: Buffer): SignedState;
export declare function getRequestFinalStatePayload(b: Buffer): Destination;
export declare function validateFinalOutcome(vFixed: FixedPart, initialOutcome: SingleAssetExit, finalOutcome: SingleAssetExit, me: Address, minAmount?: bigint): void;
export declare class ObjectiveRequest implements ObjectiveRequestInterface {
    channelId: Destination;
    private objectiveStarted?;
    constructor(params: {
        channelId?: Destination;
        objectiveStarted?: ReadWriteChannel<void>;
    });
    static newObjectiveRequest(channelId: Destination): ObjectiveRequest;
    id(address: Address, chainId?: bigint): ObjectiveId;
    waitForObjectiveToStart(): Promise<void>;
    signalObjectiveStarted(): void;
}
export declare class Objective implements ObjectiveInterface {
    status: ObjectiveStatus;
    minimumPaymentAmount?: bigint;
    v?: VirtualChannel;
    toMyLeft?: ConsensusChannel;
    toMyRight?: ConsensusChannel;
    myRole: Uint;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): Objective;
    toJSON(): any;
    constructor(params: {
        status?: ObjectiveStatus;
        minimumPaymentAmount?: bigint;
        v?: VirtualChannel;
        toMyLeft?: ConsensusChannel;
        toMyRight?: ConsensusChannel;
        myRole?: Uint;
    });
    static newObjective(request: ObjectiveRequest, preApprove: boolean, myAddress: Address, largestPaymentAmount: bigint | undefined, getChannel: GetChannelByIdFunction, getConsensusChannel: GetTwoPartyConsensusLedgerFunction): Promise<Objective>;
    static constructObjectiveFromPayload(p: ObjectivePayload, preapprove: boolean, myAddress: Address, getChannel: GetChannelByIdFunction, getTwoPartyConsensusLedger: GetTwoPartyConsensusLedgerFunction, latestVoucherAmount?: bigint): Promise<Objective>;
    private finalState;
    private initialOutcome;
    private generateFinalOutcome;
    private generateFinalState;
    id(): ObjectiveId;
    approve(): Objective;
    reject(): [Objective, SideEffects];
    ownsChannel(): Destination;
    getStatus(): ObjectiveStatus;
    related(): Storable[];
    private clone;
    private otherParticipants;
    private hasFinalStateFromAlice;
    crank(signer: NitroSigner): Promise<[Objective, SideEffects, WaitingFor]>;
    private isAlice;
    private isBob;
    private ledgerProposal;
    private updateLedgerToRemoveGuarantee;
    vId(): Destination;
    private rightHasDefunded;
    private leftHasDefunded;
    update(op: ObjectivePayload): Objective;
    receiveProposal(sp: SignedProposal): ProposalReceiver;
    private isZero;
}
export declare function isVirtualDefundObjective(id: ObjectiveId): boolean;
export declare function getVirtualChannelFromObjectiveId(id: ObjectiveId): Destination;
export {};
