/// <reference types="node" />
import { Buffer } from 'buffer';
import type { ReadWriteChannel } from '@cerc-io/ts-channel';
import { FieldDescription, NitroSigner, Uint, Uint64 } from '@cerc-io/nitro-util';
import { Destination } from '../../types/destination';
import { ConsensusChannel, SignedProposal, Proposal, Guarantee } from '../../channel/consensus-channel/consensus-channel';
import { Exit } from '../../channel/state/outcome/exit';
import { State } from '../../channel/state/state';
import { Funds } from '../../types/funds';
import { Address } from '../../types/types';
import { ObjectiveRequest as ObjectiveRequestInterface, Objective as ObjectiveInterface, ObjectiveStatus, SideEffects, WaitingFor, Storable, ProposalReceiver } from '../interfaces';
import { ObjectiveId, ObjectivePayload } from '../messages';
import { VirtualChannel } from '../../channel/virtual';
import { SignedState } from '../../channel/state/signedstate';
export declare const ObjectivePrefix = "VirtualFund-";
interface GetTwoPartyConsensusLedgerFunction {
    (counterparty: Address): [ConsensusChannel | undefined, boolean] | Promise<[ConsensusChannel | undefined, boolean]>;
}
declare class GuaranteeInfo {
    left: Destination;
    right: Destination;
    leftAmount?: Funds;
    rightAmount?: Funds;
    guaranteeDestination: Destination;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): GuaranteeInfo;
    toJSON(): any;
    constructor(params: {
        left?: Destination;
        right?: Destination;
        leftAmount?: Funds;
        rightAmount?: Funds;
        guaranteeDestination?: Destination;
    });
}
export declare class Connection {
    channel?: ConsensusChannel;
    guaranteeInfo: GuaranteeInfo;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): Connection | undefined;
    toJSON(): any;
    constructor(params: {
        channel?: ConsensusChannel;
        guaranteeInfo?: GuaranteeInfo;
    });
    insertGuaranteeInfo(a0: Funds, b0: Funds, vId: Destination, left: Destination, right: Destination): void;
    handleProposal(sp: SignedProposal): void;
    isFundingTheTarget(): boolean;
    getExpectedGuarantee(): Guarantee;
    expectedProposal(): Proposal;
}
export declare class Objective implements ObjectiveInterface, ProposalReceiver {
    status: ObjectiveStatus;
    v?: VirtualChannel;
    toMyLeft?: Connection;
    toMyRight?: Connection;
    private n;
    myRole: Uint;
    private a0?;
    private b0?;
    static jsonEncodingMap: Record<string, FieldDescription>;
    static fromJSON(data: string): Objective;
    toJSON(): any;
    constructor(params: {
        status?: ObjectiveStatus;
        v?: VirtualChannel;
        toMyLeft?: Connection;
        toMyRight?: Connection;
        n?: number;
        myRole?: number;
        a0?: Funds;
        b0?: Funds;
    });
    static newObjective(request: ObjectiveRequest, preApprove: boolean, myAddress: Address, chainId: bigint | undefined, getTwoPartyConsensusLedger: GetTwoPartyConsensusLedgerFunction): Promise<Objective>;
    static constructFromState(preApprove: boolean, initialStateOfV: State, myAddress: Address, consensusChannelToMyLeft?: ConsensusChannel, consensusChannelToMyRight?: ConsensusChannel): Objective;
    static getSignedStatePayload(b: Buffer): SignedState;
    static constructObjectiveFromPayload(p: ObjectivePayload, preapprove: boolean, myAddress: Address, getTwoPartyConsensusLedger: GetTwoPartyConsensusLedgerFunction): Promise<Objective>;
    id(): ObjectiveId;
    approve(): Objective;
    reject(): [Objective, SideEffects];
    ownsChannel(): Destination;
    getStatus(): ObjectiveStatus;
    private otherParticipants;
    private getPayload;
    receiveProposal(sp: SignedProposal): ProposalReceiver;
    update(raw: ObjectivePayload): Objective;
    crank(signer: NitroSigner): Promise<[Objective, SideEffects, WaitingFor]>;
    related(): Storable[];
    private fundingComplete;
    private clone;
    private isAlice;
    private isBob;
    private proposeLedgerUpdate;
    private acceptLedgerUpdate;
    private updateLedgerWithGuarantee;
}
export declare function isVirtualFundObjective(id: ObjectiveId): boolean;
export type ObjectiveResponse = {
    id: ObjectiveId;
    channelId: Destination;
};
export declare class ObjectiveRequest implements ObjectiveRequestInterface {
    intermediaries: Address[];
    counterParty: Address;
    challengeDuration: number;
    outcome?: Exit;
    nonce: Uint64;
    appDefinition: Address;
    private objectiveStarted?;
    constructor(params: {
        intermediaries: Address[];
        counterParty: Address;
        challengeDuration: number;
        outcome?: Exit;
        nonce: Uint64;
        appDefinition: Address;
        objectiveStarted?: ReadWriteChannel<null>;
    });
    static newObjectiveRequest(intermediaries: Address[], counterparty: Address, challengeDuration: number, outcome: Exit, nonce: Uint64, appDefinition: Address): ObjectiveRequest;
    id(myAddress: Address, chainId?: bigint): ObjectiveId;
    waitForObjectiveToStart(): Promise<void>;
    signalObjectiveStarted(): void;
    response(myAddress: Address): ObjectiveResponse;
    channelId(myAddress: Address): Destination;
}
export {};
