import { P2PMessageService } from '../node/engine/messageservice/p2p-message-service/service';
import { Node } from '../node/node';
import { Store } from '../node/engine/store/store';
import { MetricsApi } from '../node/engine/metrics';
import { Exit } from '../channel/state/outcome/exit';
import { Signature } from '../crypto/signatures';
import { ChainService } from '../node/engine/chainservice/chainservice';
/**
 * setupNode sets up a node using the given args
 *
 * @param msgPort
 * @param pk
 * @param chainPk
 */
export declare function setupNode(messageService: P2PMessageService, store: Store, chainService: ChainService, metricsApi?: MetricsApi): Promise<Node>;
/**
 * Left pads a 20 byte address hex string with zeros until it is a 32 byte hex string
 * e.g.,
 * 0x9546E319878D2ca7a21b481F873681DF344E0Df8 becomes
 * 0x0000000000000000000000009546E319878D2ca7a21b481F873681DF344E0Df8
 *
 * @param address - 20 byte hex string
 * @returns 32 byte padded hex string
 */
export declare function convertAddressToBytes32(address: string): string;
/**
 * createOutcome creates a basic outcome for a channel
 *
 * @param asset - The asset to fund the channel with
 * @param alpha - The address of the first participant
 * @param beta - The address of the second participant
 * @param amount - The amount to allocate to each participant
 * @returns An outcome for a directly funded channel with 100 wei allocated to each participant
 */
export declare function createOutcome(asset: string, alpha: string, beta: string, amount: number): Exit;
export declare function subscribeVoucherLogs(node: Node): Promise<void>;
export declare function getJoinedSignature(sig: Signature): string;
export declare const getSignerAddress: (hash: string, sig: string) => string;
